@@ -2,6 +2,7 @@ from __future__ import annotations
 
 # standard libraries
 import asyncio
+import dataclasses
 import gettext
 import re
 import typing
@@ -106,6 +107,8 @@ class DeclarativeUI:
             "size_policy_horizontal",
             "size_policy_vertical",
             "tool_tip",
+            "text_alignment_horizontal",
+            "text_alignment_vertical",
             "color",
             "font",
             "background_color",
@@ -117,9 +120,23 @@ class DeclarativeUI:
             if k in kwargs and kwargs[k] is not None:
                 d[k] = kwargs[k]
 
+    def __process_margins(self, d: typing.MutableMapping[str, typing.Any], **kwargs: typing.Any) -> None:
+        margin_properties = (
+            "margin",
+            "margin_top",
+            "margin_left",
+            "margin_bottom",
+            "margin_right",
+            "margin_horizontal",
+            "margin_vertical",
+        )
+        for k in margin_properties:
+            if k in kwargs and kwargs[k] is not None:
+                d[k] = kwargs[k]
+
     def create_column(self, *children: UIDescription, name: typing.Optional[UIIdentifier] = None,
                       items: typing.Optional[UIIdentifier] = None, item_component_id: typing.Optional[str] = None,
-                      spacing: typing.Optional[UIPoints] = None, margin: typing.Optional[UIPoints] = None,
+                      spacing: typing.Optional[UIPoints] = None,
                       **kwargs: typing.Any) -> UIDescriptionResult:
         """Create a column UI description with children or dynamic items, spacing, and margin.
 
@@ -154,18 +171,17 @@ class DeclarativeUI:
             d["item_component_id"] = item_component_id
         if spacing is not None:
             d["spacing"] = spacing
-        if margin is not None:
-            d["margin"] = margin
         if len(children) > 0:
             d_children = d.setdefault("children", list())
             for child in children:
                 d_children.append(child)
         self.__process_common_properties(d, **kwargs)
+        self.__process_margins(d, **kwargs)
         return d
 
     def create_row(self, *children: UIDescription, name: typing.Optional[UIIdentifier] = None,
                    items: typing.Optional[UIIdentifier] = None, item_component_id: typing.Optional[str] = None,
-                   spacing: typing.Optional[UIPoints] = None, margin: typing.Optional[UIPoints] = None,
+                   spacing: typing.Optional[UIPoints] = None,
                    **kwargs: typing.Any) -> UIDescriptionResult:
         """Create a row UI description with children or dynamic items, spacing, and margin.
 
@@ -200,13 +216,12 @@ class DeclarativeUI:
             d["item_component_id"] = item_component_id
         if spacing is not None:
             d["spacing"] = spacing
-        if margin is not None:
-            d["margin"] = margin
         if len(children) > 0:
             d_children = d.setdefault("children", list())
             for child in children:
                 d_children.append(child)
         self.__process_common_properties(d, **kwargs)
+        self.__process_margins(d, **kwargs)
         return d
 
     def create_spacing(self, size: UIPoints) -> UIDescriptionResult:
@@ -373,11 +388,15 @@ class DeclarativeUI:
         d: UIDescriptionResult = {"type": "scroll_area", "content": content}
         if name is not None:
             d["name"] = name
+        if "vertical_scroll_bar_policy" in kwargs:
+            d["vertical_scroll_bar_policy"] = kwargs["vertical_scroll_bar_policy"]
+        if "horizontal_scroll_bar_policy" in kwargs:
+            d["horizontal_scroll_bar_policy"] = kwargs["horizontal_scroll_bar_policy"]
         self.__process_common_properties(d, **kwargs)
         return d
 
     def create_group(self, content: UIDescription, name: typing.Optional[UIIdentifier] = None,
-                     title: typing.Optional[UILabel] = None, margin: typing.Optional[UIPoints] = None,
+                     title: typing.Optional[UILabel] = None,
                      **kwargs: typing.Any) -> UIDescriptionResult:
         """Create a group UI description with content, a name, a title, and a margin.
 
@@ -397,9 +416,8 @@ class DeclarativeUI:
             d["name"] = name
         if title is not None:
             d["title"] = title
-        if margin is not None:
-            d["margin"] = margin
         self.__process_common_properties(d, **kwargs)
+        self.__process_margins(d, **kwargs)
         return d
 
     def create_label(self, *, text: typing.Optional[UILabel] = None, name: typing.Optional[UIIdentifier] = None,
@@ -577,6 +595,35 @@ class DeclarativeUI:
         self.__process_common_properties(d, **kwargs)
         return d
 
+    def create_text_browser(self, *,
+                            text: typing.Optional[UIIdentifier] = None,
+                            markdown: typing.Optional[UIIdentifier] = None,
+                            name: typing.Optional[UIIdentifier] = None,
+                            on_anchor_clicked: typing.Optional[UICallableIdentifier] = None,
+                            **kwargs: typing.Any) -> UIDescriptionResult:
+        """Create a multi-line text browser UI description with text, name, markdown, options, and events.
+
+        Keyword Args:
+            text: handler reference to text (bindable, required)
+            text: handler reference to markdown (bindable, required)
+            name: handler property in which to store widget (optional)
+            on_anchor_clicked: callback when an anchor is clicked
+
+        Returns:
+            UI description of the text edit
+        """
+        d: UIDescriptionResult = {"type": "text_browser"}
+        if text is not None:
+            d["text"] = text
+        if markdown is not None:
+            d["markdown"] = markdown
+        if name is not None:
+            d["name"] = name
+        if on_anchor_clicked is not None:
+            d["on_anchor_clicked"] = on_anchor_clicked
+        self.__process_common_properties(d, **kwargs)
+        return d
+
     def create_push_button(self, *, text: typing.Optional[UILabel] = None, icon: typing.Optional[UIIdentifier] = None,
                            name: typing.Optional[UIIdentifier] = None,
                            on_clicked: typing.Optional[UICallableIdentifier] = None,
@@ -900,7 +947,7 @@ class DeclarativeUI:
 
     def create_modeless_dialog(self, content: UIDescription, *, title: typing.Optional[str] = None,
                                resources: typing.Optional[UIResources] = None,
-                               margin: typing.Optional[UIPoints] = None) -> UIDescriptionResult:
+                               **kwargs: typing.Any) -> UIDescriptionResult:
         """Create a modeless dialog UI description with content, title, resources, and margin.
 
         Args:
@@ -917,15 +964,15 @@ class DeclarativeUI:
         d: UIDescriptionResult = {"type": "modeless_dialog", "content": content}
         if title is not None:
             d["title"] = title
-        if margin is not None:
-            d["margin"] = margin
         if resources is not None:
             d["resources"] = resources
+        self.__process_margins(d, **kwargs)
         return d
 
     def create_window(self, content: UIDescription, *, title: typing.Optional[str] = None,
-                      resources: typing.Optional[UIResources] = None, margin: typing.Optional[UIPoints] = None,
-                      window_style: typing.Optional[str] = None) -> UIDescriptionResult:
+                      resources: typing.Optional[UIResources] = None,
+                      window_style: typing.Optional[str] = None,
+                      **kwargs: typing.Any) -> UIDescriptionResult:
         """Create a window UI description with content, title, resources, and margin.
 
         Args:
@@ -942,12 +989,11 @@ class DeclarativeUI:
         d: UIDescriptionResult = {"type": "window", "content": content}
         if title is not None:
             d["title"] = title
-        if margin is not None:
-            d["margin"] = margin
         if resources is not None:
             d["resources"] = resources
         if window_style is not None:
             d["window_style"] = window_style
+        self.__process_margins(d, **kwargs)
         return d
 
     def define_component(self, content: UIDescription, *, component_id: typing.Optional[str] = None,
@@ -1003,7 +1049,7 @@ def connect_string_value(widget: UserInterface.Widget, d: UIDescription, handler
     A value means the value for the property is directly contained in the string.
     """
     v = d.get(property)
-    m = re.match("^@binding\((.+)\)$", v if v else "")
+    m = re.match(r"^@binding\((.+)\)$", v if v else "")
     # print(f"{v}, {m}, {m.group(1) if m else 'NA'}")
     if m:
         b = m.group(1)
@@ -1067,7 +1113,7 @@ def connect_reference_value(bindable: typing.Any, d: UIDescription, handler: Han
     """
     binding_name_ = binding_name if binding_name else property
     v = d.get(property)
-    m = re.match("^@binding\((.+)\)$", v if isinstance(v, str) else "")
+    m = re.match(r"^@binding\((.+)\)$", v if isinstance(v, str) else "")
     # print(f"{v}, {m}, {m.group(1) if m else 'NA'}")
     if m:
         b = m.group(1)
@@ -1197,6 +1243,10 @@ def run_window(d: UIDescription, handler: HandlerLike, *, app: typing.Optional[A
     window = Window.Window(ui, app=app, parent_window=parent_window, persistent_id=persistent_id, window_style=window_style)
     window.title = title
     window.on_close = closer.close
+    if parent_window:
+        parent_window.register_dialog(window)
+    elif app:
+        app.register_dialog(window)
     # make and attach closer for the handler; put handler into container closer
     setattr(handler, "_closer", Closer())
     closer.push_closeable(handler)
@@ -1222,18 +1272,74 @@ def run_window(d: UIDescription, handler: HandlerLike, *, app: typing.Optional[A
     return window
 
 
-def construct_margin(ui: UserInterface.UserInterface, content: UserInterface.BoxWidget, margin: typing.Optional[int]) -> UserInterface.BoxWidget:
-    if margin:
-        column = ui.create_column_widget()
-        column.add_spacing(margin)
-        column.add(content)
-        column.add_spacing(margin)
+@dataclasses.dataclass
+class UIMargins:
+    top: typing.Optional[int] = None
+    left: typing.Optional[int] = None
+    bottom: typing.Optional[int] = None
+    right: typing.Optional[int] = None
+
+
+def construct_margins_box(ui: UserInterface.UserInterface, content: UserInterface.Widget, margins: UIMargins) -> UserInterface.BoxWidget:
+    new_content = ui.create_column_widget()
+    new_content.add(content)
+    if margins.left or margins.right:
         row = ui.create_row_widget()
-        row.add_spacing(margin)
-        row.add(column)
-        row.add_spacing(margin)
-        content = row
-    return content
+        row.add_spacing(margins.left or 0)
+        row.add(new_content)
+        row.add_spacing(margins.right or 0)
+        new_content = row
+    if margins.top or margins.bottom:
+        column = ui.create_column_widget()
+        column.add_spacing(margins.top or 0)
+        column.add(new_content)
+        column.add_spacing(margins.bottom or 0)
+        new_content = column
+    return new_content
+
+
+def construct_sizing_properties(d: UIDescription) -> UIDescriptionResult:
+    properties: UIDescriptionResult = dict()
+    for k in ("width", "min_width", "max_width", "height", "min_height", "max_height"):
+        v = d.get(k, None)
+        if v is not None:
+            properties[k.replace("_", "-")] = int(v)
+    for k in ("size_policy_horizontal", "size_policy_vertical"):
+        v = d.get(k, None)
+        if v is not None:
+            properties[k.replace("_", "-")] = str(v)
+    return properties
+
+
+def construct_margins(d: UIDescription) -> UIMargins:
+    margins = UIMargins()
+    margin = d.get("margin")
+    margin_h = d.get("margin_horizontal")
+    margin_v = d.get("margin_vertical")
+    margin_top = d.get("margin_top")
+    margin_left = d.get("margin_left")
+    margin_bottom = d.get("margin_bottom")
+    margin_right = d.get("margin_right")
+    if margin:
+        margins.top = margin
+        margins.left = margin
+        margins.bottom = margin
+        margins.right = margin
+    if margin_h:
+        margins.left = margin_h
+        margins.right = margin_h
+    if margin_v:
+        margins.top = margin_v
+        margins.bottom = margin_v
+    if margin_top:
+        margins.top = margin_top
+    if margin_left:
+        margins.left = margin_left
+    if margin_bottom:
+        margins.bottom = margin_bottom
+    if margin_right:
+        margins.right = margin_right
+    return margins
 
 
 # to properly type the container widget needs more work. substitute typing.Any for now.
@@ -1250,7 +1356,7 @@ def connect_items(ui: UserInterface.UserInterface, window: Window.Window, contai
     First it checks whether the current handler responds to `get_resource` and calls it with the `item_component_id` to
     establish a component. If not successful, it checks whether the `resources` map is defined on the current handler
     and looks up the `item_component_id`. If either of those succeed, it establishes the `component_id` and
-    `component_content` from established component. Otherwise it uses the `item_component_id` as the `component_id`
+    `component_content` from established component. Otherwise, it uses the `item_component_id` as the `component_id`
     and continues.
 
     Next, it calls `create_handler` with the established `component_id` and `item`. If the handler defines a `ui_view`,
@@ -1268,7 +1374,7 @@ def connect_items(ui: UserInterface.UserInterface, window: Window.Window, contai
     items_key = items_parts[-1]
 
     # the _closer should have been set on the handler, even if no close method is present. insert_item makes this
-    # assumption so that sub-components have a path by which to get closed.
+    # assumption so that subcomponents have a path by which to get closed.
     assert getattr(handler, "_closer")
 
     def adjust_spacing() -> None:
@@ -1309,7 +1415,7 @@ def connect_items(ui: UserInterface.UserInterface, window: Window.Window, contai
                 component_handler._closer = Closer()
                 getattr(handler, "_closer").push_closeable(component_handler)
                 component_content = getattr(component_handler, "ui_view", component_content)
-            assert component_content
+            assert component_content, f"missing component content {component_id=}"
             item_finishes: _FinishesListType = list()
             # now construct the widget
             item_widget = construct(ui, window, component_content, component_handler, item_finishes)
@@ -1347,19 +1453,6 @@ def connect_items(ui: UserInterface.UserInterface, window: Window.Window, contai
     getattr(handler, "_closer").push_closeable(container.item_removed_event.listen(row_item_removed))
 
 
-def construct_sizing_properties(d: UIDescription) -> UIDescriptionResult:
-    properties: UIDescriptionResult = dict()
-    for k in ("width", "min_width", "max_width", "height", "min_height", "max_height"):
-        v = d.get(k, None)
-        if v is not None:
-            properties[k.replace("_", "-")] = int(v)
-    for k in ("size_policy_horizontal", "size_policy_vertical"):
-        v = d.get(k, None)
-        if v is not None:
-            properties[k.replace("_", "-")] = str(v)
-    return properties
-
-
 class DeclarativeConstructor(typing.Protocol):
     def construct(self, d_type: str, ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription,
                   handler: HandlerLike, finishes: typing.Optional[_FinishesListType] = None) -> UserInterface.Widget:
@@ -1374,11 +1467,9 @@ def construct(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescr
         # this seems like a type error. is this used anywhere?
         return typing.cast(UserInterface.Widget, construct_modeless_dialog(ui, window, d, handler))
     elif d_type == "column":
-        properties = construct_sizing_properties(d)
-        return construct_box(ui, window, ui.create_column_widget(properties=properties), d, handler, finishes)
+        return construct_box(ui, window, True, d, handler, finishes)
     elif d_type == "row":
-        properties = construct_sizing_properties(d)
-        return construct_box(ui, window, ui.create_row_widget(properties=properties), d, handler, finishes)
+        return construct_box(ui, window, False, d, handler, finishes)
     elif d_type == "text_label":
         return construct_text_label(ui, d, handler, finishes)
     elif d_type == "image":
@@ -1387,6 +1478,8 @@ def construct(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescr
         return construct_line_edit(ui, d, finishes, handler)
     elif d_type == "text_edit":
         return construct_text_edit(ui, d, finishes, handler)
+    elif d_type == "text_browser":
+        return construct_text_browser(ui, d, finishes, handler)
     elif d_type == "push_button":
         return construct_push_button(ui, d, handler, finishes)
     elif d_type == "check_box":
@@ -1479,7 +1572,7 @@ def construct_list_box(ui: UserInterface.UserInterface, d: UIDescription, handle
     return widget
 
 
-def construct_group(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription, handler: HandlerLike,
+def construct_group_original(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription, handler: HandlerLike,
                     finishes: _FinishesListType) -> UserInterface.Widget:
     properties = construct_sizing_properties(d)
     widget = ui.create_group_widget(properties)
@@ -1504,11 +1597,25 @@ def construct_group(ui: UserInterface.UserInterface, window: Window.Window, d: U
     return widget
 
 
+def construct_group(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription, handler: HandlerLike,
+                    finishes: _FinishesListType) -> UserInterface.Widget:
+    properties = construct_sizing_properties(d)
+    widget = ui.create_group_widget(properties)
+    content = construct(ui, window, typing.cast(UIDescription, d.get("content")), handler, finishes)
+    margins_box = construct_margins_box(ui, content, construct_margins(d))
+    widget.add(margins_box)
+    if handler:
+        connect_name(widget, d, handler)
+        connect_string_value(widget, d, handler, "title", finishes)
+        connect_attributes(widget, d, handler, finishes)
+    return widget
+
+
 def construct_scroll_area(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription,
                           handler: HandlerLike, finishes: _FinishesListType) -> UserInterface.ScrollAreaWidget:
     properties = construct_sizing_properties(d)
     widget = ui.create_scroll_area_widget(properties)
-    widget.set_scrollbar_policies("needed", "needed")
+    widget.set_scrollbar_policies(d.get("horizontal_scroll_bar_policy", "needed"), d.get("vertical_scroll_bar_policy", "needed"))
     content = typing.cast(UIDescription, d.get("content"))
     widget.content = construct(ui, window, content, handler, finishes)
     if handler:
@@ -1683,7 +1790,10 @@ def construct_check_box(ui: UserInterface.UserInterface, d: UIDescription, handl
 def construct_push_button(ui: UserInterface.UserInterface, d: UIDescription, handler: HandlerLike,
                           finishes: _FinishesListType) -> UserInterface.PushButtonWidget:
     properties = construct_sizing_properties(d)
-    widget = ui.create_push_button_widget(properties=properties)
+    if d.get("style", None) == "minimal":
+        widget = UserInterface.PushButtonWidget(Widgets.PushButtonWidgetBehavior(ui, properties=properties), None)
+    else:
+        widget = ui.create_push_button_widget(properties=properties)
     if handler:
         connect_name(widget, d, handler)
         if d.get("text", None):
@@ -1713,6 +1823,19 @@ def construct_text_edit(ui: UserInterface.UserInterface, d: UIDescription, finis
     return widget
 
 
+def construct_text_browser(ui: UserInterface.UserInterface, d: UIDescription, finishes: _FinishesListType,
+                           handler: HandlerLike) -> UserInterface.TextBrowserWidget:
+    properties = construct_sizing_properties(d)
+    widget = ui.create_text_browser_widget(properties)
+    if handler:
+        connect_name(widget, d, handler)
+        connect_reference_value(widget, d, handler, "text", finishes)
+        connect_reference_value(widget, d, handler, "markdown", finishes)
+        connect_event(widget, widget, d, handler, "on_anchor_clicked", ["anchor"])
+        connect_attributes(widget, d, handler, finishes)
+    return widget
+
+
 def construct_line_edit(ui: UserInterface.UserInterface, d: UIDescription, finishes: _FinishesListType,
                         handler: HandlerLike) -> UserInterface.LineEditWidget:
     editable = d.get("editable", None)
@@ -1756,16 +1879,23 @@ def construct_text_label(ui: UserInterface.UserInterface, d: UIDescription, hand
         connect_string_value(widget, d, handler, "text", finishes)
         connect_name(widget, d, handler)
         connect_attributes(widget, d, handler, finishes)
+        connect_reference_value(widget, d, handler, "text_alignment_horizontal", finishes, binding_name="text_alignment_horizontal", value_type=str)
+        connect_reference_value(widget, d, handler, "text_alignment_vertical", finishes, binding_name="text_alignment_vertical", value_type=str)
         connect_reference_value(widget, d, handler, "color", finishes, binding_name="text_color", value_type=str)
         connect_reference_value(widget, d, handler, "font", finishes, binding_name="text_font", value_type=str)
         connect_reference_value(widget, d, handler, "word_wrap", finishes, value_type=bool)
     return widget
 
 
-def construct_box(ui: UserInterface.UserInterface, window: Window.Window, box_widget: UserInterface.BoxWidget,
+def construct_box(ui: UserInterface.UserInterface, window: Window.Window, is_column: bool,
                   d: UIDescription, handler: HandlerLike, finishes: _FinishesListType) -> UserInterface.BoxWidget:
+    sizing_properties = construct_sizing_properties(d)
+    if is_column:
+        box_widget = ui.create_column_widget(properties=sizing_properties)
+    else:
+        box_widget = ui.create_row_widget(properties=sizing_properties)
+    margins_box_widget = construct_margins_box(ui, box_widget, construct_margins(d))
     spacing: typing.Optional[int] = d.get("spacing")
-    margin: typing.Optional[int] = d.get("margin")
     items = d.get("items")
     item_component_id = d.get("item_component_id")
     children = d.get("children", list())
@@ -1785,13 +1915,12 @@ def construct_box(ui: UserInterface.UserInterface, window: Window.Window, box_wi
         connect_items(ui, window, box_widget, handler, items, item_component_id, spacing_v=spacing)
     if handler:
         connect_name(box_widget, d, handler)
-        connect_attributes(box_widget, d, handler, finishes)
-    return construct_margin(ui, box_widget, margin)
+        connect_attributes(margins_box_widget, d, handler, finishes)
+    return margins_box_widget
 
 
 def construct_modeless_dialog(ui: UserInterface.UserInterface, window: Window.Window, d: UIDescription, handler: HandlerLike) -> Dialog.ActionDialog:
     title = d.get("title", _("Untitled"))
-    margin = d.get("margin")
     persistent_id = d.get("persistent_id")
     content = typing.cast(UIDescription, d.get("content"))
     resources = d.get("resources", dict())
@@ -1809,18 +1938,8 @@ def construct_modeless_dialog(ui: UserInterface.UserInterface, window: Window.Wi
     # make and attach closer for the handler; put handler into container closer
     setattr(handler, "_closer", Closer())
     closer.push_closeable(handler)
-    outer_row = ui.create_row_widget()
-    outer_column = ui.create_column_widget()
-    inner_content = construct(ui, window, content, handler, finishes)
-    if margin is not None:
-        outer_row.add_spacing(margin)
-        outer_column.add_spacing(margin)
-    outer_column.add(inner_content)
-    outer_row.add(outer_column)
-    if margin is not None:
-        outer_row.add_spacing(margin)
-        outer_column.add_spacing(margin)
-    dialog.content.add(outer_row)
+    content_widget = construct(ui, window, content, handler, finishes)
+    dialog.content.add(construct_margins_box(ui, content_widget, construct_margins(d)))
     for finish in finishes:
         finish()
     setattr(handler, "_event_loop", window.event_loop)
@@ -1837,7 +1956,7 @@ class ComponentWidget(Widgets.CompositeWidgetBase):
         self.__window = window
         self.__handler = handler
         self.__d = d
-        self.__component_handler = None
+        self.__component_handler: typing.Optional[HandlerLike] = None
 
         def set_identifier(value: typing.Optional[str]) -> None:
             self.__update_identifier(value)
@@ -1885,27 +2004,28 @@ class ComponentWidget(Widgets.CompositeWidgetBase):
             component_id = identifier
         if component_id:
             # create the handler first, but don't initialize it.
-            component_handler = getattr(handler, "create_handler")(component_id=component_id) if component_id and hasattr(handler, "create_handler") else None
-            if component_handler:
-                # make and attach closer for the component handler and link it to the container handler.
-                component_handler._closer = Closer()
-                getattr(handler, "_closer").push_closeable(component_handler)
-                component_content = getattr(component_handler, "ui_view", component_content)
-                # set properties in the component from the properties dict
-                for k, v in d.get("properties", dict()).items():
-                    # print(f"setting property {k} to {v}")
-                    setattr(component_handler, k, v)
-                self.__component_handler = component_handler
-            assert component_content
+            component_handler = typing.cast(HandlerLike, getattr(handler, "create_handler")(component_id=component_id))
+            # make and attach closer for the component handler and link it to the container handler.
+            setattr(component_handler, "_closer", Closer())
+            getattr(handler, "_closer").push_closeable(component_handler)
+            component_content = getattr(component_handler, "ui_view", component_content)
+            assert component_content, f"{component_handler} missing 'ui_view'."
+            # set properties in the component from the properties dict
+            for k, v in d.get("properties", dict()).items():
+                # print(f"setting property {k} to {v}")
+                setattr(component_handler, k, v)
+            self.__component_handler = component_handler
+            assert component_content, f"{component_id} not found."
             # now construct the widget
             widget = construct(ui, window, component_content, component_handler, finishes)
             # connect the name to the handler if desired
             connect_name(widget, d, handler)
             # since the handler is custom to the widget, make a way to retrieve it from the widget
             setattr(widget, "handler", component_handler)
-            component_handler._event_loop = window.event_loop
-            if callable(getattr(component_handler, "init_handler", None)):
-                component_handler.init_handler()
+            setattr(component_handler, "_event_loop", window.event_loop)
+            init_handler = getattr(component_handler, "init_handler", None)
+            if callable(init_handler):
+                init_handler()
             # connect events
             for event in events:
                 # print(f"connecting {event['event']} ({event['parameters']})")
