@@ -35,10 +35,10 @@ from nion.utils import Geometry
 # pylint: disable=star-args
 
 
-RGBA32Type = typing.Any
-RGBA8Type = typing.Any
-GrayscaleF32Type = typing.Any
-C8Type = typing.Any
+RGBA32Type = numpy.typing.NDArray[numpy.uint32]
+RGBA8Type = numpy.typing.NDArray[numpy.uint8]
+GrayscaleF32Type = numpy.typing.NDArray[numpy.float32]
+C8Type = numpy.typing.NDArray[numpy.uint8]
 
 ByteOrderType = typing.Optional[typing.Union[typing.Literal["little"], typing.Literal["big"]]]
 
@@ -99,6 +99,24 @@ def get_alpha_view(rgba_image: RGBA32Type, byteorder: ByteOrderType = None) -> C
         return typing.cast(C8Type, bytes[..., 0])  # A of ARGB
 
 
+class LinearGradient:
+    next = 1
+
+    def __init__(self, width: float, height: float, x1: float, y1: float, x2: float, y2: float) -> None:
+        self.commands: typing.List[typing.Sequence[typing.Any]] = []
+        self.binary_commands = bytearray()
+        self.command_var = LinearGradient.next
+        self.commands.append(("gradient", self.command_var, float(width), float(height), float(x1), float(y1), float(x2), float(y2)))
+        self.binary_commands.extend(struct.pack("4siffffff", b"grad", self.command_var, float(width), float(height), float(x1), float(y1), float(x2), float(y2)))
+        LinearGradient.next += 1
+
+    def add_color_stop(self, x: float, color: typing.Optional[str]) -> None:
+        color = color or "#0000"
+        self.commands.append(("colorStop", self.command_var, float(x), str(color)))
+        color_encoded = color.encode("utf-8")
+        self.binary_commands.extend(struct.pack("4sifi{}s0i".format(len(color_encoded)), b"grcs", self.command_var, float(x), len(color_encoded), color_encoded))
+
+
 class DrawingContext:
     """
         Path commands (begin_path, close_path, move_to, line_to, etc.) should not be intermixed
@@ -115,7 +133,7 @@ class DrawingContext:
         self.commands: typing.List[typing.Sequence[typing.Any]] = []
         self.binary_commands = bytearray()
         self.save_count = 0
-        self.images: typing.Dict[str, RGBA32Type] = dict()
+        self.images: typing.Dict[str, typing.Union[RGBA32Type, GrayscaleF32Type]] = dict()
 
     def copy_from(self, drawing_context: DrawingContext) -> None:
         assert self.save_count == 0
@@ -224,6 +242,7 @@ class DrawingContext:
 
     def to_svg(self, size: Geometry.IntSize, viewbox: Geometry.IntRect) -> str:
         svg = ""
+        style = "text { font-family: sans-serif; }"
         defs = ""
         path = ""
         next_clip_id = 1
@@ -253,7 +272,7 @@ class DrawingContext:
         def parse_color(color_str: str) -> typing.Tuple[str, float]:
             color_str = ''.join(color_str.split())
             if color_str.startswith("rgba"):
-                c = re.split("rgba\((\d+),(\d+),(\d+),([\d.]+)\)", color_str)
+                c = re.split(r"rgba\((\d+),(\d+),(\d+),([\d.]+)\)", color_str)
                 return f"rgb({c[1]}, {c[2]}, {c[3]})", float(c[4])
             return color_str, 1.0
 
@@ -350,11 +369,10 @@ class DrawingContext:
                 rgba_data = get_rgba_view_from_rgba_data(image)
                 # image compression is time consuming. pass parameters to make this step as fast as possible.
                 # see nionswift-642.
-                # TODO: fix typing when imageio gets their numpy typing correct.
-                imageio.imwrite(png_file, rgba_data[..., (2, 1, 0, 3)], extension=".png", optimize=False, compress_level=1)  # type: ignore
+                imageio.imwrite(png_file, rgba_data[..., (2, 1, 0, 3)], extension=".png", optimize=False, compress_level=1)
                 png_encoded = base64.b64encode(png_file.getvalue()).decode('utf=8')
                 transform_str = " transform='{0}'".format(" ".join(transform)) if len(transform) > 0 else ""
-                svg_format_str = "<image x='{0}' y='{1}' width='{2}' height='{3}' xlink:href='data:image/png;base64,{4}'{5} />"
+                svg_format_str = "<image x='{0}' y='{1}' width='{2}' height='{3}' style='image-rendering:pixelated' xlink:href='data:image/png;base64,{4}'{5} />"
                 svg += svg_format_str.format(a, b, c, d, png_encoded, transform_str)
             elif command_id == "data":
                 w, h, data, data_id, a, b, c, d, low, high, color_table, color_table_image_id = command_args
@@ -381,11 +399,10 @@ class DrawingContext:
                 png_file = io.BytesIO()
                 # image compression is time consuming. pass parameters to make this step as fast as possible.
                 # see nionswift-642.
-                # TODO: fix typing when imageio gets their numpy typing correct.
-                imageio.imwrite(png_file, get_rgba_view_from_rgba_data(image), extension=".png", optimize=False, compress_level=1)  # type: ignore
+                imageio.imwrite(png_file, get_rgba_view_from_rgba_data(image), extension=".png", optimize=False, compress_level=1)
                 png_encoded = base64.b64encode(png_file.getvalue()).decode('utf=8')
                 transform_str = " transform='{0}'".format(" ".join(transform)) if len(transform) > 0 else ""
-                svg_format_str = "<image x='{0}' y='{1}' width='{2}' height='{3}' xlink:href='data:image/png;base64,{4}'{5} />"
+                svg_format_str = "<image x='{0}' y='{1}' width='{2}' height='{3}' style='image-rendering:pixelated' xlink:href='data:image/png;base64,{4}'{5} />"
                 svg += svg_format_str.format(a, b, c, d, png_encoded, transform_str)
             elif command_id == "stroke":
                 if stroke_style is not None:
@@ -484,6 +501,7 @@ class DrawingContext:
                                                                                                             size.height,
                                                                                                             viewbox_str,
                                                                                                             xmlns)
+        result += "<style>" + style + "</style>"
         result += "<defs>" + defs + "</defs>"
         result += svg
         result += "</svg>"
@@ -585,8 +603,14 @@ class DrawingContext:
         self.binary_commands.extend(struct.pack("4sffff", b"quad", float(x1), float(y1), float(x), float(y)))
 
     def draw_image(self, img: RGBA32Type, x: float, y: float, width: float, height: float) -> None:
+        # The host application expects the image to be contiguous c-style memory. Check and update here if required.
+        if not img.flags['C_CONTIGUOUS']:
+            img = numpy.ascontiguousarray(img)
         # img should be rgba pack, uint32
         assert img.dtype == numpy.uint32
+        # The numpy-2 compatible nionui-tool takes an object which has the bytearray interface. The hdf5 dataset does
+        # not - so convert it to a numpy array. The conversion should be a no-op if	the image is already a numpy array.
+        img = numpy.asarray(img, numpy.uint32)
         with DrawingContext.__image_id_lock:
             DrawingContext.__image_id += 1
             image_id = DrawingContext.__image_id
@@ -596,8 +620,16 @@ class DrawingContext:
         self.binary_commands.extend(struct.pack("4siiiffff", b"imag", img.shape[1], img.shape[0], int(image_id), float(x), float(y), float(width), float(height)))
 
     def draw_data(self, img: GrayscaleF32Type, x: float, y: float, width: float, height: float, low: float, high: float, color_map_data: typing.Optional[RGBA32Type]) -> None:
+        # The host application expects the image to be contiguous c-style memory. Check and update here if required.
+        if not img.flags['C_CONTIGUOUS']:
+            img = numpy.ascontiguousarray(img)
+        if color_map_data is not None and color_map_data.flags['C_CONTIGUOUS']:
+            color_map_data = numpy.ascontiguousarray(color_map_data)
         # img should be float
         assert img.dtype == numpy.float32
+        # The numpy-2 compatible nionui-tool takes an object which has the bytearray interface. The hdf5 dataset does
+        # not - so convert it to a numpy array. The conversion should be a no-op if	the image is already a numpy array.
+        img = numpy.asarray(img, numpy.float32)
         with DrawingContext.__image_id_lock:
             DrawingContext.__image_id += 1
             image_id = DrawingContext.__image_id
@@ -655,7 +687,7 @@ class DrawingContext:
     @fill_style.setter
     def fill_style(self, a: typing.Optional[typing.Union[str, LinearGradient]]) -> None:
         a = a or "rgba(0, 0, 0, 0.0)"
-        if isinstance(a, DrawingContext.LinearGradient):
+        if isinstance(a, LinearGradient):
             self.commands.extend(a.commands)
             self.commands.append(("fillStyleGradient", int(a.command_var)))
             self.binary_commands.extend(a.binary_commands)
@@ -770,24 +802,8 @@ class DrawingContext:
         a_encoded = a.encode("utf-8")
         self.binary_commands.extend(struct.pack("4si{}s0i".format(len(a_encoded)), b"lnjn", len(a_encoded), a_encoded))
 
-    class LinearGradient:
-        next = 1
-
-        def __init__(self, width: float, height: float, x1: float, y1: float, x2: float, y2: float) -> None:
-            self.commands: typing.List[typing.Sequence[typing.Any]] = []
-            self.binary_commands = bytearray()
-            self.command_var = DrawingContext.LinearGradient.next
-            self.commands.append(("gradient", self.command_var, float(width), float(height), float(x1), float(y1), float(x2), float(y2)))
-            self.binary_commands.extend(struct.pack("4siffffff", b"grad", self.command_var, float(width), float(height), float(x1), float(y1), float(x2), float(y2)))
-            DrawingContext.LinearGradient.next += 1
-
-        def add_color_stop(self, x: float, color: str) -> None:
-            self.commands.append(("colorStop", self.command_var, float(x), str(color)))
-            color_encoded = color.encode("utf-8")
-            self.binary_commands.extend(struct.pack("4sifi{}s0i".format(len(color_encoded)), b"grcs", self.command_var, float(x), len(color_encoded), color_encoded))
-
-    def create_linear_gradient(self, width: float, height: float, x1: float, y1: float, x2: float, y2: float) -> DrawingContext.LinearGradient:
-        gradient = DrawingContext.LinearGradient(width, height, x1, y1, x2, y2)
+    def create_linear_gradient(self, width: float, height: float, x1: float, y1: float, x2: float, y2: float) -> LinearGradient:
+        gradient = LinearGradient(width, height, x1, y1, x2, y2)
         return gradient
 
     def statistics(self, stat_id: str) -> None:
