@@ -11,6 +11,7 @@ import typing
 import numpy
 
 # local libraries
+from . import Bitmap
 from . import CanvasItem
 from . import DrawingContext
 from . import UserInterface as UserInterfaceModule
@@ -400,6 +401,10 @@ class Key(UserInterfaceModule.Key):
     def is_tab(self) -> bool:
         return self.__key == "tab"
 
+    @property
+    def is_backtab(self) -> bool:
+        return self.__key == "backtab"
+
     @property
     def is_arrow(self) -> bool:
         return self.is_left_arrow or self.is_up_arrow or self.is_right_arrow or self.is_down_arrow
@@ -530,7 +535,7 @@ class WidgetBehavior:
     def periodic(self) -> None:
         pass
 
-    def _set_root_container(self, window: typing.Optional[Window.Window]) -> None:
+    def _set_root_container(self, window: typing.Optional[UserInterfaceModule.Window]) -> None:
         pass
 
     def _get_content_widget(self) -> typing.Optional[UserInterfaceModule.Widget]:
@@ -555,7 +560,10 @@ class WidgetBehavior:
             if self.on_focus_changed:
                 self.on_focus_changed(focused)
 
-    def set_background_color(self, color: typing.Optional[str]) -> None:
+    def set_background_color(self, color: typing.Optional[typing.Union[str, DrawingContext.LinearGradient]]) -> None:
+        pass
+
+    def set_border_color(self, value: typing.Optional[str]) -> None:
         pass
 
     def set_property(self, key: str, value: typing.Any) -> None:
@@ -567,7 +575,7 @@ class WidgetBehavior:
     def map_to_global(self, p: Geometry.IntPoint) -> Geometry.IntPoint:
         return p
 
-    def drag(self, mime_data: UserInterfaceModule.MimeData, thumbnail: typing.Optional[DrawingContext.RGBA32Type] = None,
+    def drag(self, mime_data: UserInterfaceModule.MimeData, thumbnail: typing.Optional[Bitmap.Bitmap] = None,
              hot_spot_x: typing.Optional[int] = None, hot_spot_y: typing.Optional[int] = None,
              drag_finished_fn: typing.Optional[typing.Callable[[str], None]] = None) -> None:
         pass
@@ -593,7 +601,7 @@ class NullBehavior:
     def periodic(self) -> None:
         pass
 
-    def _set_root_container(self, window: typing.Optional[Window.Window]) -> None:
+    def _set_root_container(self, window: typing.Optional[UserInterfaceModule.Window]) -> None:
         pass
 
     def _get_content_widget(self) -> typing.Optional[UserInterfaceModule.Widget]:
@@ -605,12 +613,15 @@ class NullBehavior:
     def map_to_global(self, p: Geometry.IntPoint) -> Geometry.IntPoint:
         return Geometry.IntPoint()
 
-    def drag(self, mime_data: UserInterfaceModule.MimeData, thumbnail: typing.Optional[DrawingContext.RGBA32Type] = None,
+    def drag(self, mime_data: UserInterfaceModule.MimeData, thumbnail: typing.Optional[Bitmap.Bitmap] = None,
              hot_spot_x: typing.Optional[int] = None, hot_spot_y: typing.Optional[int] = None,
              drag_finished_fn: typing.Optional[typing.Callable[[str], None]] = None) -> None:
         pass
 
-    def set_background_color(self, value: typing.Optional[str]) -> None:
+    def set_background_color(self, value: typing.Optional[typing.Union[str, DrawingContext.LinearGradient]]) -> None:
+        pass
+
+    def set_border_color(self, value: typing.Optional[str]) -> None:
         pass
 
 
@@ -666,6 +677,9 @@ class BoxWidgetBehavior(WidgetBehavior):
         self.widget.children.insert(index, child_widget)
         child_widget.size_changed(self.widget.size)
 
+    def remove(self, child: UserInterfaceModule.Widget) -> None:
+        pass
+
     def add_stretch(self) -> UserInterfaceModule.Widget:
         self.__widgets.append(WidgetItem(WidgetItemType.STRETCH))
         self.widget.children.append(Widget("stretch"))
@@ -735,7 +749,7 @@ class StackWidgetBehavior(WidgetBehavior):
 
     def __init__(self, widget_type: str, properties: typing.Optional[typing.Mapping[str, typing.Any]]) -> None:
         super().__init__(widget_type, properties)
-        self.current_index = -1
+        self.current_index: typing.Optional[int] = None
         self.__children: typing.List[UserInterfaceModule.Widget] = list()
 
     def close(self) -> None:
@@ -849,7 +863,8 @@ class PushButtonWidgetBehavior(WidgetBehavior):
     def __init__(self, widget_type: str, properties: typing.Optional[typing.Mapping[str, typing.Any]]) -> None:
         super().__init__(widget_type, properties)
         self.text: typing.Optional[str] = None
-        self.icon: typing.Optional[DrawingContext.RGBA32Type] = None
+        self.icon: typing.Optional[Bitmap.Bitmap] = None
+        self.icon_shape: typing.Optional[Geometry.IntSize] = None
         self.on_clicked: typing.Optional[typing.Callable[[], None]] = None
 
 
@@ -858,7 +873,7 @@ class RadioButtonWidgetBehavior(WidgetBehavior):
     def __init__(self, widget_type: str, properties: typing.Optional[typing.Mapping[str, typing.Any]]) -> None:
         super().__init__(widget_type, properties)
         self.text: typing.Optional[str] = None
-        self.icon: typing.Optional[DrawingContext.RGBA32Type] = None
+        self.icon: typing.Optional[Bitmap.Bitmap] = None
         self.checked = False
         self.on_clicked: typing.Optional[typing.Callable[[], None]] = None
 
@@ -880,6 +895,12 @@ class LabelWidgetBehavior(WidgetBehavior):
         self.text: typing.Optional[str] = None
         self.word_wrap = False
 
+    def set_text_alignment_horizontal(self, alignment: typing.Optional[str]) -> None:
+        pass
+
+    def set_text_alignment_vertical(self, alignment: typing.Optional[str]) -> None:
+        pass
+
     def set_text_color(self, color: typing.Optional[str]) -> None:
         pass
 
@@ -951,6 +972,9 @@ class CanvasWidgetBehavior(WidgetBehavior):
     def _set_canvas_item(self, canvas_item: CanvasItem.AbstractCanvasItem) -> None:
         pass
 
+    def _create_composition_canvas_item(self, canvas_widget: UserInterfaceModule.CanvasWidget, layout_render: typing.Optional[str]) -> CanvasItem.CanvasWidgetCanvasItem:
+        return CanvasItem.RootCanvasItem(canvas_widget, layout_render=layout_render)
+
     def periodic(self) -> None:
         pass
 
@@ -989,6 +1013,9 @@ class CanvasWidgetBehavior(WidgetBehavior):
     def show_tool_tip_text(self, text: str, gx: int, gy: int) -> None:
         pass
 
+    def hide_tool_tip_text(self) -> None:
+        pass
+
 
 class LineEditWidgetBehavior(WidgetBehavior):
 
@@ -1026,6 +1053,47 @@ class LineEditWidgetBehavior(WidgetBehavior):
             self.on_editing_finished(text)
 
 
+class TextBrowserWidgetBehavior(WidgetBehavior):
+
+    def __init__(self, widget_type: str, properties: typing.Optional[typing.Mapping[str, typing.Any]]) -> None:
+        super().__init__(widget_type, properties)
+        self.on_anchor_clicked: typing.Optional[typing.Callable[[str], None]] = None
+        self.on_load_image_resource: typing.Optional[typing.Callable[[str], typing.Optional[DrawingContext.RGBA32Type]]] = None
+        self.on_escape_pressed: typing.Optional[typing.Callable[[], bool]] = None
+        self.on_return_pressed: typing.Optional[typing.Callable[[], bool]] = None
+        self.on_key_pressed: typing.Optional[typing.Callable[[UserInterfaceModule.Key], bool]] = None
+        self._no_focus = "click_focus"
+
+    def close(self) -> None:
+        self.on_anchor_clicked = None
+        self.on_load_image_resource = None
+        self.on_escape_pressed = None
+        self.on_return_pressed = None
+        self.on_key_pressed = None
+        super().close()
+
+    def set_html(self, value: typing.Optional[str]) -> None:
+        pass
+
+    def set_markdown(self, value: typing.Optional[str]) -> None:
+        pass
+
+    def set_text(self, value: typing.Optional[str]) -> None:
+        pass
+
+    def set_text_background_color(self, color: typing.Optional[str]) -> None:
+        pass
+
+    def set_text_color(self, color: typing.Optional[str]) -> None:
+        pass
+
+    def set_text_font(self, font_str: typing.Optional[str]) -> None:
+        pass
+
+    def scroll_to_anchor(self, value: str) -> None:
+        pass
+
+
 class TextEditWidgetBehavior(WidgetBehavior):
 
     def __init__(self, widget_type: str, properties: typing.Optional[typing.Mapping[str, typing.Any]]) -> None:
@@ -1183,21 +1251,29 @@ class DocumentWindow(UserInterfaceModule.Window):
     def show(self, size: typing.Optional[Geometry.IntSize] = None, position: typing.Optional[Geometry.IntPoint] = None) -> None:
         pass
 
+    def activate(self) -> None:
+        pass
+
     def restore(self, geometry: str, state: str) -> None:
         pass
 
     def _get_focus_widget(self) -> typing.Optional[UserInterfaceModule.Widget]:
-        return None
-        # wrong type. disabling.
-        # global focused_widget
-        # return focused_widget
+        def match_native_widget(widget: typing.Optional[UserInterfaceModule.Widget]) -> typing.Optional[UserInterfaceModule.Widget]:
+            if widget and widget.focused:
+                return widget
+            for child_widget in (widget._contained_widgets if widget else list()):
+                matched_widget = match_native_widget(child_widget)
+                if matched_widget:
+                    return matched_widget
+            return None
+        return match_native_widget(self.root_widget)
 
 
 class DockWidget(UserInterfaceModule.DockWidget):
 
     def __init__(self, window: DocumentWindow, widget: UserInterfaceModule.Widget, panel_id: str, title: str, positions: typing.Sequence[str], position: str) -> None:
         super().__init__(window, widget, panel_id, title, positions, position)
-        self.visible = False
+        self.__visible = False
         self.__focus_policy = "no_focus"
         self.does_retain_focus = False
 
@@ -1207,12 +1283,16 @@ class DockWidget(UserInterfaceModule.DockWidget):
         action.on_ui_activity = self._register_ui_activity
         return action
 
+    @property
+    def visible(self) -> bool:
+        return self.__visible
+
     def show(self) -> None:
-        self.visible = True
+        self.__visible = True
         self._register_ui_activity()
 
     def hide(self) -> None:
-        self.visible = False
+        self.__visible = False
         self._register_ui_activity()
 
     def size_changed(self, size: Geometry.IntSize) -> None:
@@ -1304,6 +1384,9 @@ class UserInterface(UserInterfaceModule.UserInterface):
     def create_line_edit_widget(self, properties: typing.Optional[typing.Mapping[str, typing.Any]] = None) -> UserInterfaceModule.LineEditWidget:
         return UserInterfaceModule.LineEditWidget(LineEditWidgetBehavior("lineedit", properties))
 
+    def create_text_browser_widget(self, properties: typing.Optional[typing.Mapping[str, typing.Any]] = None) -> UserInterfaceModule.TextBrowserWidget:
+        return UserInterfaceModule.TextBrowserWidget(TextBrowserWidgetBehavior("textbrowser", properties))
+
     def create_text_edit_widget(self, properties: typing.Optional[typing.Mapping[str, typing.Any]] = None) -> UserInterfaceModule.TextEditWidget:
         return UserInterfaceModule.TextEditWidget(TextEditWidgetBehavior("textedit", properties))
 
@@ -1397,6 +1480,9 @@ class UserInterface(UserInterfaceModule.UserInterface):
     def get_qt_version(self) -> str:
         return "TEST"
 
+    def get_build_version(self) -> str:
+        return "BUILD"
+
     def get_tolerance(self, tolerance_type: UserInterfaceModule.ToleranceType) -> float:
         return 5
 
