@@ -3,6 +3,7 @@ A basic class to serve as the document controller of a typical one window applic
 """
 from __future__ import annotations
 
+import abc
 # standard libraries
 import asyncio
 import enum
@@ -17,7 +18,7 @@ import weakref
 from nion.utils import Event
 from nion.utils import Geometry
 from nion.utils import Process
-from nion.ui import DrawingContext
+from nion.ui import Bitmap
 from nion.ui import UserInterface
 
 if typing.TYPE_CHECKING:
@@ -94,8 +95,6 @@ class Action:
     action_id: str = str()
     action_name: str = str()
     action_role: typing.Optional[str] = None
-    action_summary: typing.Optional[str] = None
-    action_description: typing.Optional[str] = None
     action_parameters: typing.List[ActionProperty] = list()
 
     def __init__(self) -> None:
@@ -110,7 +109,7 @@ class Action:
 
     def event(self, context: ActionContext) -> ActionResult:
         """Handle the event."""
-        ...
+        return ActionResult(ActionStatus.PASS)
 
     def execute(self, context: ActionContext) -> ActionResult:
         """Execute the action with the context. No user interaction allowed. May be called from scripts."""
@@ -129,6 +128,9 @@ class Action:
     def get_action_name(self, context: ActionContext) -> str:
         return self.action_name
 
+    def get_action_summary(self, context: ActionContext) -> str:
+        return f"{self.action_id}: {self.action_name}"
+
     def report(self, type: ReportType, message: str) -> None:
         self.__reports.append(Report(type, message))
 
@@ -152,7 +154,7 @@ actions: typing.Mapping[str, Action] = dict()
 
 
 def register_action(action: Action) -> None:
-    assert action.action_id not in actions
+    assert action.action_id not in actions, f"{action.action_id}"
     typing.cast(typing.MutableMapping[str, Action], actions)[action.action_id] = action
 
 
@@ -230,6 +232,7 @@ class Window:
         self.__document_window.on_position_changed = self.position_changed
         self.__document_window.on_refocus_widget = self.refocus_widget
         self.__document_window.on_ui_activity = self._register_ui_activity
+        self.__document_window.on_physical_dpi_changed = self.__physical_dpi_changed
         self.__periodic_queue = Process.TaskQueue()
         self.__periodic_set = Process.TaskSet()
         self.__modal_actions: typing.List[Action] = list()
@@ -366,12 +369,16 @@ class Window:
             self.periodic()
 
     def periodic(self) -> None:
-        self.__periodic_queue.perform_tasks()
-        self.__periodic_set.perform_tasks()
-        self.__event_loop.stop()
-        self.__event_loop.run_forever()
+        with Process.audit("perform_task_queue"):
+            self.__periodic_queue.perform_tasks()
+        with Process.audit("perform_task_set"):
+            self.__periodic_set.perform_tasks()
+        with Process.audit("perform_async"):
+            self.__event_loop.stop()
+            self.__event_loop.run_forever()
         if self.app:
-            self.app.periodic()
+            with Process.audit("app_periodic"):
+                self.app.periodic()
         # if the request close flag is set, request the close.
         # this must be at the end of periodic since it will result in the window
         # close method being called.
@@ -481,7 +488,7 @@ class Window:
             self._adjust_menus()
         return False
 
-    def drag(self, mime_data: UserInterface.MimeData, thumbnail: typing.Optional[DrawingContext.RGBA32Type], hot_spot_x: int, hot_spot_y: int) -> None:
+    def drag(self, mime_data: UserInterface.MimeData, thumbnail: typing.Optional[Bitmap.BitmapOrArray], hot_spot_x: int, hot_spot_y: int) -> None:
         root_widget = self.__document_window.root_widget
         if root_widget:
             root_widget.drag(mime_data, thumbnail, hot_spot_x, hot_spot_y)
@@ -529,6 +536,11 @@ class Window:
     def display_scaling(self) -> float:
         return self.__document_window.display_scaling
 
+    def __physical_dpi_changed(self, physical_dpi: float) -> None:
+        root_widget = self.__document_window.root_widget
+        if root_widget:
+            root_widget.redraw()
+
     def get_font_metrics(self, font: str, text: str) -> UserInterface.FontMetrics:
         return self.ui.get_font_metrics(font, text)
 
@@ -549,6 +561,7 @@ class Window:
 
     def show(self, *, size: typing.Optional[Geometry.IntSize] = None, position: typing.Optional[Geometry.IntPoint] = None) -> None:
         self.__document_window.show(size=size, position=position)
+        self.__document_window.activate()
 
     def activate(self) -> None:
         self.__document_window.activate()
@@ -577,14 +590,12 @@ class Window:
     # added tasks do not guarantee execution order or execution at all.
 
     def add_task(self, key: str, task: typing.Callable[[], None]) -> None:
-        assert task
         self.__periodic_set.add_task(key + str(id(self)), task)
 
     def clear_task(self, key: str) -> None:
         self.__periodic_set.clear_task(key + str(id(self)))
 
     def queue_task(self, task: typing.Callable[[], None]) -> None:
-        assert task
         self.__periodic_queue.put(task)
 
     def clear_queued_tasks(self) -> None:
@@ -628,7 +639,13 @@ class Window:
                 action_id = item_d["action_id"]
                 action = actions.get(action_id)
                 if action:
-                    key_sequence = action_shortcuts.get(action_id, dict()).get("window")
+                    action_shortcut_d = action_shortcuts.get(action_id, dict())
+                    key_sequence: str | None = None
+                    for context_key in action_shortcut_d.keys():
+                        if action_shortcut_d[context_key]:
+                            key_sequences = [action_shortcut_d[context_key]] if isinstance(action_shortcut_d[context_key], str) else action_shortcut_d[context_key]
+                            key_sequence = key_sequences[0]  # only use the first shortcut for menus
+                            break
                     role = getattr(action, "about_role", None)
                     assert menu is not None
                     menu.add_menu_item(action.action_name, functools.partial(self.perform_action, action_id),
@@ -716,13 +733,17 @@ class Window:
             action = action_or_action_id
         else:
             action = actions.get(str(action_or_action_id))
-        if action and action not in self.__modal_actions:
+        if action and action not in self.__modal_actions and action.is_enabled(action_context):
             action.clear()
-            if action.invoke(action_context).status == ActionStatus.MODAL:
+            action_status = self._invoke_action_with_context(action, action_context).status
+            if action_status == ActionStatus.MODAL:
                 self.__modal_actions.append(action)
             for report in action.reports:
                 self.display_report(report)
 
+    def _invoke_action_with_context(self, action: Action, action_context: ActionContext) -> ActionResult:
+        return action.invoke(action_context)
+
     def display_report(self, report: Report) -> None:
         from nion.ui import Dialog  # avoid circular reference
         if report.type == ReportType.DEBUG:
