@@ -29,7 +29,7 @@ class TestCanvasItemClass(unittest.TestCase):
         widget = Widgets.StringListWidget(ui)
         with contextlib.closing(widget):
             canvas_item = typing.cast(CanvasItem.CanvasItemComposition, typing.cast(UserInterface.CanvasWidget, typing.cast(UserInterface.BoxWidget, widget.content_widget).children[0]).canvas_item)
-            canvas_item.update_layout(Geometry.IntPoint(x=0, y=0), Geometry.IntSize(width=300, height=200), immediate=True)
+            canvas_item.update_layout(Geometry.IntPoint(x=0, y=0), Geometry.IntSize(width=300, height=200))
             scroll_area_canvas_item = typing.cast(CanvasItem.ScrollAreaCanvasItem, typing.cast(CanvasItem.CanvasItemComposition, canvas_item.canvas_items[0]).canvas_items[0])
             canvas_item.layout_immediate(Geometry.IntSize(width=300, height=200))
             # check assumptions
@@ -40,11 +40,9 @@ class TestCanvasItemClass(unittest.TestCase):
             scroll_content_rect = scroll_content.canvas_rect or Geometry.IntRect.empty_rect()
             self.assertEqual(scroll_content_rect.height, 0)
             # add item
-            self.assertFalse(canvas_item._needs_layout_for_testing)
             widget.items = ["abc"]
-            # self.assertTrue(canvas_item._needs_layout_for_testing)
             # check that column was laid out again
-            canvas_item.layout_immediate(Geometry.IntSize(width=300, height=200), force=False)
+            canvas_item.layout_immediate(Geometry.IntSize(width=300, height=200))
             scroll_canvas_rect = scroll_area_canvas_item.canvas_rect or Geometry.IntRect.empty_rect()
             scroll_content = scroll_area_canvas_item.content
             assert scroll_content
